"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  BrowserProvider: () => BrowserProvider,
  LocalStorage: () => LocalStorage,
  SessionStorage: () => SessionStorage,
  StorageProvider: () => StorageProvider
});
module.exports = __toCommonJS(src_exports);

// src/adapter/WebStorage.ts
var import_utils = require("@stenodb/utils");
var BrowserStorage = class {
  name;
  storage;
  logger;
  entityTransformer;
  dataTransformer;
  data = null;
  initialData = null;
  constructor(name, storage, entity, initialData) {
    this.name = name;
    this.storage = storage;
    this.entityTransformer = (0, import_utils.entityTransformer)(entity);
    this.dataTransformer = (0, import_utils.dataTransformer)(this.entityTransformer);
    if (initialData) {
      this.initialData = initialData;
    }
  }
  registerLogger(logger) {
    this.logger = logger;
  }
  read() {
    const data = this.storage.getItem(this.name);
    this.data = this.dataTransformer.toJSON(data);
    this.logger?.info("Read data:", this.data);
  }
  write() {
    this.storage.setItem(this.name, this.dataTransformer.toString(this.data));
    this.logger?.info("Write data:", this.data);
  }
  reset() {
    if (!this.initialData)
      return;
    this.data = this.entityTransformer(this.initialData);
    this.write();
  }
  exists() {
    return this.read() !== null;
  }
};

// src/adapter/LocalStorage.ts
var LocalStorage = class extends BrowserStorage {
  constructor(name, entity, initialData) {
    super(name, localStorage, entity, initialData);
  }
};

// src/adapter/SessionStorage.ts
var SessionStorage = class extends BrowserStorage {
  constructor(name, entity, initialData) {
    super(name, sessionStorage, entity, initialData);
  }
};

// src/provider/StorageProvider.ts
var StorageProvider = class {
  #adapter;
  constructor(adapter) {
    this.#adapter = adapter;
  }
  get data() {
    return this.#adapter.data;
  }
  set data(data) {
    this.#adapter.data = data;
  }
  get initialData() {
    return this.#adapter.initialData;
  }
  set initialData(data) {
    if (!data)
      return;
    this.#adapter.initialData = data;
  }
  read() {
    this.#adapter.read();
    if (!this.#adapter.data) {
      this.reset();
    }
    return this.#adapter.data;
  }
  write() {
    this.#adapter.write();
  }
  reset() {
    this.#adapter.reset();
  }
  exists() {
    return this.#adapter.exists();
  }
};

// src/provider/BrowserProvider.ts
var BrowserProvider = class {
  #options;
  constructor(options = {}) {
    this.#options = options;
  }
  registerAdapterModules(adapter) {
    if (!this.#options?.logger)
      return;
    const logger = this.#options.logger(adapter.name);
    adapter.registerLogger(logger);
  }
  create(adapter) {
    this.registerAdapterModules(adapter);
    return new StorageProvider(adapter);
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  BrowserProvider,
  LocalStorage,
  SessionStorage,
  StorageProvider
});
