import { BaseLogger, CreateLogger } from '@stenodb/logger';
import { EntityTransformer, DataTransformer, ClassEntity } from '@stenodb/utils';

declare class BrowserStorage<T> {
    name: string;
    storage: Storage;
    logger: BaseLogger | undefined;
    entityTransformer: EntityTransformer<T>;
    dataTransformer: DataTransformer<T>;
    data: T | null;
    initialData: T | null;
    constructor(name: string, storage: Storage, entity: ClassEntity<T>, initialData?: T);
    registerLogger(logger: BaseLogger): void;
    read(): void;
    write(): void;
    reset(): void;
    exists(): boolean;
}

declare class LocalStorage<T> extends BrowserStorage<T> {
    constructor(name: string, entity: ClassEntity<T>, initialData?: T);
}

declare class SessionStorage<T> extends BrowserStorage<T> {
    constructor(name: string, entity: ClassEntity<T>, initialData?: T);
}

type BrowserStorageAdapter<T> = LocalStorage<T> | SessionStorage<T>;
interface BrowserProviderOptions {
    logger?: CreateLogger;
}

declare class StorageProvider<T> {
    #private;
    constructor(adapter: BrowserStorageAdapter<T>);
    get data(): T | null;
    set data(data: T | null);
    get initialData(): T | null;
    set initialData(data: T | undefined | null);
    read(): T | null;
    write(): void;
    reset(): void;
    exists(): boolean;
}

declare class BrowserProvider {
    #private;
    constructor(options?: BrowserProviderOptions);
    private registerAdapterModules;
    create<T>(adapter: BrowserStorageAdapter<T>): StorageProvider<T>;
}

export { BrowserProvider, BrowserProviderOptions, BrowserStorageAdapter, LocalStorage, SessionStorage, StorageProvider };
