// src/adapter/WebStorage.ts
import { dataTransformer, entityTransformer } from "@stenodb/utils";
var BrowserStorage = class {
  name;
  storage;
  logger;
  entityTransformer;
  dataTransformer;
  data = null;
  initialData = null;
  constructor(name, storage, entity, initialData) {
    this.name = name;
    this.storage = storage;
    this.entityTransformer = entityTransformer(entity);
    this.dataTransformer = dataTransformer(this.entityTransformer);
    if (initialData) {
      this.initialData = initialData;
    }
  }
  registerLogger(logger) {
    this.logger = logger;
  }
  read() {
    const data = this.storage.getItem(this.name);
    this.data = this.dataTransformer.toJSON(data);
    this.logger?.info("Read data:", this.data);
  }
  write() {
    this.storage.setItem(this.name, this.dataTransformer.toString(this.data));
    this.logger?.info("Write data:", this.data);
  }
  reset() {
    if (!this.initialData)
      return;
    this.data = this.entityTransformer(this.initialData);
    this.write();
  }
  exists() {
    return this.read() !== null;
  }
};

// src/adapter/LocalStorage.ts
var LocalStorage = class extends BrowserStorage {
  constructor(name, entity, initialData) {
    super(name, localStorage, entity, initialData);
  }
};

// src/adapter/SessionStorage.ts
var SessionStorage = class extends BrowserStorage {
  constructor(name, entity, initialData) {
    super(name, sessionStorage, entity, initialData);
  }
};

// src/provider/StorageProvider.ts
var StorageProvider = class {
  #adapter;
  constructor(adapter) {
    this.#adapter = adapter;
  }
  get data() {
    return this.#adapter.data;
  }
  set data(data) {
    this.#adapter.data = data;
  }
  get initialData() {
    return this.#adapter.initialData;
  }
  set initialData(data) {
    if (!data)
      return;
    this.#adapter.initialData = data;
  }
  read() {
    this.#adapter.read();
    if (!this.#adapter.data) {
      this.reset();
    }
    return this.#adapter.data;
  }
  write() {
    this.#adapter.write();
  }
  reset() {
    this.#adapter.reset();
  }
  exists() {
    return this.#adapter.exists();
  }
};

// src/provider/BrowserProvider.ts
var BrowserProvider = class {
  #options;
  constructor(options = {}) {
    this.#options = options;
  }
  registerAdapterModules(adapter) {
    if (!this.#options?.logger)
      return;
    const logger = this.#options.logger(adapter.name);
    adapter.registerLogger(logger);
  }
  create(adapter) {
    this.registerAdapterModules(adapter);
    return new StorageProvider(adapter);
  }
};
export {
  BrowserProvider,
  LocalStorage,
  SessionStorage,
  StorageProvider
};
